const eMusic_PrefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
const eMusic_Branch = eMusic_PrefService.getBranch("eMusicToolbar.");
const maxSavedSearches = 5;

var eMusic_TB_ShowDailyDownload;
var eMusic_TB_ShowCharts;
var eMusic_TB_ShowNew;
var eMusic_TB_ShowLive;
var eMusic_TB_ShowBrowse;
var eMusic_TB_ShowRadio;
var eMusic_TB_ShowPopup;
var eMusic_TB_ShowHighlight;
var eMusic_TB_AutoComplete;
var eMusic_TB_NewWindow;
var eMusic_TB_SearchHistory;
var eMusic_TB_AutoSearch;
var eMusic_TB_AutoDragSearch;
var eMusic_TB_AutoUpdate;
var eMusic_TB_ShowText;
var eMusic_TB_ShowWordFind;
var eMusic_TB_SearcheMusic;
var eMusic_TB_SavedSearches = new Array(maxSavedSearches);

function eMusic_TrimString(string)
{
	// Return empty if nothing was passed in
	if (!string) return "";
	
	// Efficiently replace any leading or trailing whitespace
	var value = string.replace(/^\s+/, '');
	value = string.replace(/\s+$/, '');
	
	// Replace any multiple whitespace characters with a single space
	value = value.replace(/\s+/g, ' ');
	
	// Return the altered string
	return value;
}

function eMusic_ConvertTermsToURI(terms)
{
	// Split up the search terms based on the space character
	var termArray = new Array();
	termArray = terms.split(" ");
	var result = "";

	// Loop through the search terms, building up the result string
	for(var i = 0; i < termArray.length; i++)
	{
		if(i > 0) 
		{ 
			result += "+"; 
		}
		
		result += encodeURIComponent(termArray[i]);
	}

	// Return the result
	return result;
}

function eMusic_LoadURL(URL)
{
	// Set the document's location to the incoming URL
	window._content.document.location = URL;
	
	// Make sure we get the focus
	window.content.focus();
}

function eMusic_Populate()
{
	// Get the menu element that we will be working with
	var menu = document.getElementById("eMusic-SearchTermsMenu");

	// Clean up whatever is currently in the menu
	for(var i = menu.childNodes.length - 1; i>=0; i--) 
	{
		menu.removeChild(menu.childNodes.item(i));
	}

	// Load the search terms into our menu
	for(var i=0; i < maxSavedSearches; i++)
	{
		// For each search term, create a menu item element
		var tempItem = null;
		tempItem = document.createElement("menuitem");

		// Set the menuitem element's various attributes:
		// The label will be the search term itself
		if (eMusic_TB_SavedSearches[i])
		{
			tempItem.setAttribute("label", eMusic_TB_SavedSearches[i]);
			tempItem.setAttribute("oncommand", "eMusic_Search(event, 'default');");

			// Add the item to our menu
			menu.appendChild(tempItem);
		}
		else
		{
			eMusic_TB_SavedSearches[i] = "";
			eMusic_Branch.setCharPref("savedsearch"+i, eMusic_TB_SavedSearches[i]);
		}
	}
}

function eMusic_Search(event, type)
{

	// Create a few variables
	var URL = "";
	var isEmpty = false;
	var firstMatch = maxSavedSearches;
	var LastSavedSearch = maxSavedSearches;

	// Get a handle to our search terms box
	var searchTermsBox = document.getElementById("eMusic-SearchTerms");

	// Get a handle to our default search menu
	var searchMenu = document.getElementById("eMusic-SearchMenu");
    
	// Get the value in the search terms box, trimming whitespace as necessary
	var searchTerms = eMusic_TrimString(searchTermsBox.value);

	if(type == "default")
	{
		if(eMusic_TB_SearcheMusic)
			type = "eMusic";
		else
			type = "Mamma";
	}


	// If there are no search terms, than we set isEmpty to true
	// Otherwise, we convert the search terms to a safe URI version
	if(searchTerms.length == 0)
	{
		isEmpty = true;
	}
	else
	{
		for(var i = 0; i < maxSavedSearches; i++)
		{
			if( !eMusic_TB_SavedSearches[i])
			{
				LastSavedSearch = i-1;
				break;
			}	
				
			if(searchTerms == eMusic_TB_SavedSearches[i])
			{
				firstMatch = i;
				break;
			}			
		}

		for(var i = Math.min(firstMatch, (LastSavedSearch+1)); i > 0; i--)
		{
			eMusic_TB_SavedSearches[i] = eMusic_TB_SavedSearches[i-1];
			eMusic_Branch.setCharPref("savedsearch"+i, eMusic_TB_SavedSearches[i]);
		}

		eMusic_TB_SavedSearches[0] = searchTerms;
		eMusic_Branch.setCharPref("savedsearch0", eMusic_TB_SavedSearches[0]);
		
		searchTerms = eMusic_ConvertTermsToURI(searchTerms);
	}
	
	// Do something different for differing search types
	switch(type)
	{
	// Build up the URL for a web search
	case "Mamma":
		if(isEmpty)
			URL = "http://www.mamma.com/psearch.html";
		else        
			URL = "http://partners.mamma.com/MammaPSearch?cb=EmusicT&query=" + searchTerms;
		eMusic_Branch.setBoolPref("options.searchemusic", false);
		eMusic_TB_SearcheMusic = false;
		searchMenu.setAttribute("label", "Mamma Web Search");
		break;
	
	// Build up the URL for an eMusic search
	case "eMusic":
	default:
		if(isEmpty)
			URL = "http://www.emusic.com/search.html";
		else
			URL = "http://www.emusic.com/search.html?mode=x&QT=" + searchTerms;
		eMusic_Branch.setBoolPref("options.searchemusic", true);
		eMusic_TB_SearcheMusic = true;
		searchMenu.setAttribute("label", "eMusic Search");
		break;
	}
	
	// Load the URL in the browser window using the eMusic_LoadURL function
	eMusic_LoadURL(URL);
}
